@@ -30,9 +30,9 @@ from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord
 from Bio.Data import PDBData, IUPACData
 
-from .seqio import save_fasta, read_seqres, Fasta
-from .typedef import FilePathType, FilePathOrIOType, StructureFragmentAAType, StructureFragmentType
-from .utils import ensure_path, ensure_fileio
+from protools.seqio import save_fasta, read_seqres, Fasta
+from protools.typedef import FilePathType, FilePathOrIOType, StructureFragmentAAType, StructureFragmentType, SeqLikeType
+from protools.utils import ensure_path, ensure_fileio, ensure_seq_string
 from tqdm.auto import tqdm
 
 
@@ -204,7 +204,7 @@ def _write_seqres(target_path: FilePathOrIOType, seqres: dict):
     format_line = lambda k: 'SEQRES {:>3d} {:1s} {:>4d} ' + ''.join([' {:>3s}'] * k)
     f, need_close = ensure_fileio(target_path, 'w')
     for chain, seq in seqres.items():
-        if isinstance(seq, (str, Seq)):
+        if isinstance(seq, SeqLikeType):
             seq = [PDBData.protein_letters_1to3.get(aa, 'UNK') for aa in seq]
         elif isinstance(seq, (list, tuple)):
             pass
@@ -463,7 +463,7 @@ def read_pdb_seq(entity: StructureFragmentType, standard: bool = True, unknown_a
     """
     Extract the sequence of a S.
 
-    Parameters
+    Yields
     ----------
     entity : StructureFragmentType
         Structure, Model or Chain object.
@@ -472,10 +472,9 @@ def read_pdb_seq(entity: StructureFragmentType, standard: bool = True, unknown_a
     unknown_aa : str, optional
         Symbol to represent unknown amino acids, by default 'X'.
 
-    Returns
-    ----------
-    seq_iter : Iterable[Tuple[str, str, Seq]]
-        An iterable of tuples (model_id, chain_id, seq).
+    Yields
+    ------
+    An tuples (model_id, chain_id, seq).
     """
     if entity.level not in ['S', 'M', 'C']:
         raise ValueError("Require Structure, Model or Chain object")
@@ -574,20 +573,29 @@ def pdb2seq(path: FilePathType) -> Fasta:
     fasta : Fasta
         Fasta object of the PDB file.
     """
-    seqs = read_seqres(path)
-    if len(seqs) == 0:
-        seqs = read_pdb_seq(get_structure(path))
-        seqs = Fasta((x[1], x[2]) for x in seqs)
+    path = ensure_path(path)
+    if path.is_dir():
+        paths = filter(lambda x: x.suffix.lower() in ['.pdb', '.cif'] and x.is_file, path.iterdir())
+    else:
+        paths = [path]
+    seqs = Fasta()
+    for p in paths:
+        s = read_seqres(p)
+        if len(s) == 0:
+            s = read_pdb_seq(get_structure(p))
+            s = ((x[1], x[2]) for x in s)
+        else:
+            s = ((k, v.seq) for k, v in s.items())
+        seqs.update({f"{p.stem}_{k}": v for k, v in s})
     return seqs
 
-
-def pdb2df(entity: Union[Structure, Model, Chain, Residue], *extra_attrs: str) -> pd.DataFrame:
+def pdb2df(entity: StructureFragmentAAType, *extra_attrs: str) -> pd.DataFrame:
     """
     Convert an entity to a pandas DataFrame.
 
     Parameters
     ----------
-    entity : Union[Structure, Model, Chain, Residue]
+    entity : StructureFragmentAAType
         Entity to be converted.
 
     extra_attrs : str
@@ -669,7 +677,7 @@ def read_residue(pdb: Union[FilePathType, str, Entity], mode='centroid') -> pd.D
     """
     if isinstance(pdb, Entity):
         structure = pdb
-    elif isinstance(pdb, (str, Path)):
+    elif isinstance(pdb, FilePathType):
         structure = get_structure(pdb)
     else:
         raise TypeError(f"Unsupported type {type(pdb)}")
@@ -852,7 +860,7 @@ def generate_missing_atoms_remarks(data: pd.DataFrame) -> Dict[int, list]:
 
 
 def coord2chain(
-        coord: np.ndarray, seq: str, chain_id: str = 'A', atoms: List[str] = None,
+        coord: np.ndarray, seq: SeqLikeType, chain_id: str = 'A', atoms: List[str] = None,
         init_residue_number: int = 1, init_serial_number: int = 1) -> Chain:
     """
     Convert coordinates to a Chain object. It's useful to build a new structure
@@ -862,7 +870,7 @@ def coord2chain(
     ----------
     coord : np.ndarray
         Coordinates of the atoms.
-    seq : str
+    seq : SeqLikeType
         Sequence of the chain.
     chain_id : str, optional
         ID of the chain, by default 'A'.
@@ -882,6 +890,7 @@ def coord2chain(
     if atoms is None:
         atoms = BACKBONE_ATOMS
     chain = Chain(chain_id)
+    seq = ensure_seq_string(seq)
     coord = coord.reshape(len(seq), len(atoms), 3).astype(np.float32)
     serial_number = init_serial_number
     for res_idx, res_letter in enumerate(seq):
