@@ -8,9 +8,9 @@ import logging
 import warnings
 
 from pathlib import Path
-from typing import Optional, List, Callable, Generator
+from typing import Dict, Iterable, Optional, List, Callable
 from io import IOBase
-from .typedef import FilePathType, FilePathOrIOType
+from protools.typedef import FilePathType, FilePathOrIOType, SeqLikeType
 from collections import namedtuple
 from contextlib import contextmanager
 
@@ -18,7 +18,7 @@ from contextlib import contextmanager
 AsyncCompletedProcess = namedtuple('AsyncCompletedProcess', ['stdout', 'stderr'])
 
 
-def ensure_path(path: FilePathType) -> Path:
+def ensure_path(path: FilePathType, mk_parents: bool = False) -> Path:
     """
     Convert the input to a Path object.
 
@@ -26,6 +26,8 @@ def ensure_path(path: FilePathType) -> Path:
     ----------
     path : FilePathType
         The input path.
+    mk_parents : bool, optional
+        Whether to create the parent directories if they do not exist, by default False.
 
     Returns
     ----------
@@ -34,6 +36,8 @@ def ensure_path(path: FilePathType) -> Path:
     """
     if not isinstance(path, Path):
         path = Path(path)
+    if mk_parents:
+        path.parent.mkdir(parents=True, exist_ok=True)
     return path.expanduser().resolve().absolute()
 
 
@@ -64,8 +68,8 @@ def ensure_fileio(path_or_io: FilePathOrIOType, mode: str = 'r') -> IOBase:
     ValueError
         If the input is a closed IO object or the mode is not matched.
     """
-    if isinstance(path_or_io, (str, Path)):
-        return ensure_path(path_or_io).open(mode), True
+    if isinstance(path_or_io, FilePathType):
+        return ensure_path(path_or_io, mk_parents=True).open(mode), True
     elif isinstance(path_or_io, IOBase):
         if path_or_io.closed:
             raise ValueError('The input handler is closed.')
@@ -76,6 +80,27 @@ def ensure_fileio(path_or_io: FilePathOrIOType, mode: str = 'r') -> IOBase:
         raise TypeError(f'Unsupported type: {type(path_or_io)}')
 
 
+def ensure_seq_string(seq: SeqLikeType) -> str:
+    """
+    Ensure the input is a valid seq object and convert it to string.
+
+    Parameters
+    ----------
+    seq : SeqLikeType (str, Seq, SeqRecord)
+        The input sequence-like object.
+    Returns
+    ----------
+    str
+        The string representation of the sequence.
+    """
+    if not isinstance(seq, SeqLikeType):
+        raise TypeError(
+            f'Input should be a sequence-like object, got {type(seq)}')
+    if isinstance(seq, str):
+        return seq
+    return ''.join(seq)
+
+
 def max_retry(max: int = 3, err_types: Exception =(RuntimeError,), interval: int = 1):
     def decorator(func):
         @functools.wraps(func)
@@ -136,6 +161,23 @@ class CmdWrapperBase(object):
         Whether the boolean value will be converted to flag.
         If it is True, the boolean argument will be converted
         to a flag without value.
+        For example, `flag=True` will be converted to `--flag`,
+        and `flag=False` will be ignored.
+        Default is False.
+    list2nargs : bool, optional
+        Whether the list or tuple value will be converted to
+        multiple arguments.
+        If it is True, the list or tuple argument will be converted
+        to multiple arguments.
+        For example, `arg=[1, 2, 3]` will be converted
+        to `--arg 1 2 3`.
+    underline2hyphen : bool, optional
+        Whether the underline in the argument name will be
+        converted to hyphen.
+        If it is True, the underline in the argument name will be
+        converted to hyphen.
+        For example, `arg_name` will be converted to `--arg-name`.
+        Default is True.
     """
     def __init__(self, 
                  cmd: str,
@@ -143,12 +185,14 @@ class CmdWrapperBase(object):
                  num_workers: int = 1,
                  install_help: Optional[str] = None,
                  list2nargs: bool = True,
-                 bool2flag: bool = False):
+                 bool2flag: bool = False,
+                 underline2hyphen: bool = False):
         self.cmd = self.find_command(cmd, install_help)
         self.short_mode = short_mode
         self.semaphore = asyncio.Semaphore(num_workers)
         self.bool2flag = bool2flag
         self.list2nargs = list2nargs
+        self.underline2hyphen = underline2hyphen
 
     @classmethod
     def _check_mod(cls, cmd) -> str:
@@ -198,6 +242,8 @@ class CmdWrapperBase(object):
         cmds = [self.cmd]
         cmds.extend(map(str, args))
         for k, v in kwargs.items():
+            if self.underline2hyphen:
+                k = k.replace('_', '-')
             if len(k) == 1 or self.short_mode:
                 cmds.append(f'-{k}')
             else:
@@ -214,12 +260,13 @@ class CmdWrapperBase(object):
     
     def __call__(
             self, 
-            *args, 
-            stdout=None,
-            stderr=None,
-            stdin=None,
-            cwd=None,
-            env=None,
+            *args,
+            pipe_input: Optional[bytes] = None,
+            stdout: Optional[IOBase] = None,
+            stderr: Optional[IOBase] = None,
+            stdin: Optional[IOBase] = None,
+            cwd: Optional[FilePathType] = None,
+            env: Optional[Dict[str, str]] = None,
             **kwargs):
         """
         Run the command.
@@ -228,6 +275,8 @@ class CmdWrapperBase(object):
         ----------
         *args
             The positional arguments.
+        pipe_input
+            The input to be passed to the command's stdin.
         stdout
             The stdout of the command. See subprocess.run for more details.
         stderr
@@ -243,7 +292,8 @@ class CmdWrapperBase(object):
         """
         cmds = self._format_args(*args, **kwargs)
         process = subprocess.run(
-            cmds, 
+            cmds,
+            input=pipe_input,
             stdout=stdout, 
             stderr=stderr, 
             stdin=stdin,
@@ -255,9 +305,12 @@ class CmdWrapperBase(object):
     async def async_call(
             self, 
             *args,
-            stdout=None,
-            stderr=None,
-            stdin=None,
+            pipe_input: Optional[bytes] = None,
+            stdout: Optional[IOBase] = None,
+            stderr: Optional[IOBase] = None,
+            stdin: Optional[IOBase] = None,
+            cwd: Optional[FilePathType] = None,
+            env: Optional[Dict[str, str]] = None,
             **kwargs) -> AsyncCompletedProcess:
         """
         Run the command asynchronously.
@@ -268,12 +321,18 @@ class CmdWrapperBase(object):
         ----------
         *args
             The positional arguments.
+        pipe_input
+            The input to be passed to the command's stdin.
         stdout
             The stdout of the command. See subprocess.run for more details.
         stderr
             The stderr of the command. See subprocess.run for more details.
         stdin
             The stdin of the command. See subprocess.run for more details.
+        cwd
+            The current working directory of the command.
+        env
+            The environment variables of the command.
         **kwargs
             The keyword arguments.
 
@@ -282,14 +341,22 @@ class CmdWrapperBase(object):
         AsyncCompletedProcess
             The result of the command.
         """
+        # asyncio subprocess not check pipe_input and stdin, 
+        # so we need to check them here
+        if pipe_input is not None:
+            if stdin is not None:
+                raise ValueError('stdin and pipe_input cannot be set at the same time.')
+            stdin = subprocess.PIPE
         async with self.semaphore:
             cmds = self._format_args(*args, **kwargs)
             process = await asyncio.create_subprocess_exec(
                 *cmds, 
                 stdout=stdout, 
                 stderr=stderr, 
-                stdin=stdin)
-            async_stdout, async_stderr = await process.communicate()
+                stdin=stdin,
+                cwd=cwd,
+                env=env)
+            async_stdout, async_stderr = await process.communicate(input=pipe_input)
             return AsyncCompletedProcess(async_stdout, async_stderr)
     
     def __getattr__(self, name: str):
@@ -318,9 +385,13 @@ class CmdWrapperBase(object):
                 f"'{self.__class__.__name__}' object has no attribute '{name}'")
         if name.startswith('async_'):
             name = name[6:]
+            if self.underline2hyphen:
+                name = name.replace('_', '-')
             async def wrapper(*args, **kwargs):
                 return await self.async_call(name, *args, **kwargs)
         else:
+            if self.underline2hyphen:
+                name = name.replace('_', '-')
             def wrapper(*args, **kwargs):
                 return self(name, *args, **kwargs)
         return wrapper
@@ -504,13 +575,13 @@ class Intervals(object):
             raise ValueError("Dynamic interval does not have length")
         return sum([i.stop - i.start for i in self._interval_slices])
 
-    def create_view_symbol(self, seq: str, mark_symbol: str = '*', other_symbol: str = ' ') -> str:
+    def create_view_symbol(self, seq: SeqLikeType, mark_symbol: str = '*', other_symbol: str = ' ') -> str:
         """
         Create a view symbol for the intervals in a sequence.
 
         Parameters
         ----------
-        seq : str
+        seq : SeqLikeType
             The sequence to create the view symbol.
         mark_symbol : str, optional
             The symbol to mark the intervals. Default is '*'.
@@ -522,6 +593,7 @@ class Intervals(object):
         str
             The view symbol of the intervals in the sequence.
         """
+        seq = ensure_seq_string(seq)
         seq_len = len(seq)
         marks = [other_symbol] * seq_len
         for interval in self._interval_slices:
@@ -596,7 +668,7 @@ class Intervals(object):
         new_interval = new_interval.intersect(bound_interval)
         return new_interval
 
-    def iterpos(self) -> Generator[int, None, None]:
+    def iterpos(self) -> Iterable[int]:
         """
         Iterate the 0-based positions in the intervals.
 
